Package: GridOnClusters
Type: Package
Title: Multivariate Joint Grid Discretization
Version: 0.3.2
Date: 2025-12-12
Depends: R (>= 3.5.0)
Authors@R: 
    c(person(given = "Jiandong",
             family = "Wang",
             role = "aut",
             email = "wangjd24@nmsu.edu"),
      person(given = "Sajal",
             family = "Kumar",
             role = "aut",
             email = "sajal49@nmsu.edu",
             comment = c(ORCID = "0000-0003-0930-1582")),
      person(given = "Joe",
             family = "Song",
             role = c("aut", "cre"),
             email = "joemsong@nmsu.edu",
             comment = c(ORCID = "0000-0002-6883-6547")))
Author: Jiandong Wang [aut],
  Sajal Kumar [aut] (ORCID: <https://orcid.org/0000-0003-0930-1582>),
  Joe Song [aut, cre] (ORCID: <https://orcid.org/0000-0002-6883-6547>)
Maintainer: Joe Song <joemsong@nmsu.edu>
Description: Discretize multivariate continuous data using a grid
 to capture the joint distribution that preserves clusters in
 original data. It can handle both labeled or unlabeled data.
 Both published methods (Wang et al 2020) <doi:10.1145/3388440.3412415>
 and new methods are included. Joint grid discretization
 can prepare data for model-free inference of association,
 function, or causality.
Imports: Rcpp, Ckmeans.1d.dp, cluster, fossil, dqrng, mclust, Rdpack,
        plotrix
Suggests: FunChisq, knitr, testthat (>= 2.1.0), rmarkdown
RdMacros: Rdpack
License: LGPL (>= 3)
Encoding: UTF-8
LinkingTo: BH, Rcpp
RoxygenNote: 7.3.3
NeedsCompilation: yes
VignetteBuilder: knitr
Packaged: 2025-12-12 13:19:38 UTC; joesong
Repository: CRAN
Date/Publication: 2025-12-12 13:40:07 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 09:53:27 UTC; unix
Archs: GridOnClusters.so.dSYM
