# Generated by roxygen2: do not edit by hand

S3method(plot,GridOnClusters)
export(ballclust)
export(cluster)
export(discretize.jointly)
export(gen_simdata)
export(hashed_upsilon)
export(plotGOCpatterns)
import(Rcpp)
importFrom(Ckmeans.1d.dp,MultiChannel.WUC)
importFrom(Ckmeans.1d.dp,ahist)
importFrom(Rdpack,reprompt)
importFrom(cluster,pam)
importFrom(cluster,silhouette)
importFrom(dqrng,dqsample)
importFrom(fossil,adj.rand.index)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,palette)
importFrom(graphics,abline)
importFrom(graphics,legend)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,strheight)
importFrom(graphics,strwidth)
importFrom(mclust,Mclust)
importFrom(mclust,hc)
importFrom(mclust,mclustBIC)
importFrom(plotrix,color.legend)
importFrom(stats,dist)
importFrom(stats,kmeans)
importFrom(stats,pchisq)
importFrom(stats,rnorm)
useDynLib(GridOnClusters)
