# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

test_upsilon_by_hashing <- function() {
    invisible(.Call('_GridOnClusters_test_upsilon_by_hashing', PACKAGE = 'GridOnClusters'))
}

fast_VII_mclust <- function(X, K_range, threads = 1L, min_pts = 5L, verbose = FALSE) {
    .Call('_GridOnClusters_fast_VII_mclust', PACKAGE = 'GridOnClusters', X, K_range, threads, min_pts, verbose)
}

median_distance_c <- function(data, labels) {
    .Call('_GridOnClusters_median_distance_c', PACKAGE = 'GridOnClusters', data, labels)
}

mean_distance_c <- function(data, labels) {
    .Call('_GridOnClusters_mean_distance_c', PACKAGE = 'GridOnClusters', data, labels)
}

findgrid <- function(cluster_info, k, nobs, ndims, min_bins, max_bins, method, cutoff = 0.05, entropy = FALSE, reduction = FALSE, n_thread = 1L) {
    .Call('_GridOnClusters_findgrid', PACKAGE = 'GridOnClusters', cluster_info, k, nobs, ndims, min_bins, max_bins, method, cutoff, entropy, reduction, n_thread)
}

upsilon_c <- function(data, log) {
    .Call('_GridOnClusters_upsilon_c', PACKAGE = 'GridOnClusters', data, log)
}

