% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ClusterVII.R
\name{ballclust}
\alias{ballclust}
\title{Fast VII Gaussian Mixture Model}
\usage{
ballclust(
  data,
  G = NULL,
  threads = parallel::detectCores() - 1,
  min_pts = 5,
  verbose = FALSE
)
}
\arguments{
\item{data}{A numeric matrix or data frame.}

\item{G}{An integer vector specifying the numbers of mixture components. 
If NULL, defaults to 1:9.}

\item{threads}{Number of threads for OpenMP parallelization.}

\item{min_pts}{an integer to specify the minimum number of points inside 
a cluster. Default: 5}

\item{verbose}{logical. if \code{TRUE}, print out additional messages. 
Default: \code{FALSE}}
}
\value{
An object of class \code{ballclust}, mimicking \code{mclust}.
}
\description{
Fast VII Gaussian Mixture Model
}
