% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/discretize_jointly.R
\name{cluster}
\alias{cluster}
\title{Cluster Multivariate Data}
\usage{
cluster(data, k, method, noise, min_pts, nthread)
}
\arguments{
\item{data}{input continuous multivariate data}

\item{k}{the number(s) of clusters}

\item{method}{the method for clustering}

\item{noise}{adding jitter noise to the data or not}

\item{min_pts}{an integer to specify the minimum number of points inside a cluster.}

\item{nthread}{an integer to specify the number of threads}
}
\description{
The function obtains clusters from data using the given
  number of clusters, which may be a range.
}
