% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{hashed_upsilon}
\alias{hashed_upsilon}
\title{Hashed Upsilon Test of Association}
\usage{
hashed_upsilon(points, log.p = FALSE)
}
\arguments{
\item{points}{a matrix of observed categorical data points. 
Rows are observations and columns are variables.}

\item{log.p}{a logical. If \code{TRUE}, the returned p.value is 
logarithm of the true p-value. Useful for improved precision
when true p-values are close to zero.}
}
\description{
Hashed Upsilon Test of Association
}
\seealso{
\link[Upsilon]{upsilon.test} for full description of 
  the Upsilon test.
}
